/*
 * @package netNotes
 * @author Sebastian Tschan
 * @copyright (c) Sebastian Tschan
 * @license GNU General Public License
 * @link https://blueimp.net/mozilla/
 */

const EXPORTED_SYMBOLS = ['Notes'];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import('resource://gre/modules/utils.js');
Cu.import('resource://netNotes/constants.js');

const AS = PlacesUtils.annotations;
const BS = PlacesUtils.bookmarks;
const HS = PlacesUtils.history;
const TS = PlacesUtils.tagging;

const EXPIRE_NEVER = AS.EXPIRE_NEVER;
const TYPE_FOLDER = BS.TYPE_FOLDER;
			
function Notes(bookmarksFolderID, pageNoteTags) {
	// Make sure the given bookmarksFolder is valid:
	if(!bookmarksFolderID || !this._isBookmarkFolder(bookmarksFolderID)) {
		this.bookmarksFolderID = BS.unfiledBookmarksFolder;
	} else {
		this.bookmarksFolderID = bookmarksFolderID;
	}
	this.pageNoteTags = pageNoteTags ? pageNoteTags : null;
}

Notes.prototype = {

	getPageNote: function(uri) {
		// Get the most recently added/modified bookmark for a URL,
		// excluding items under tag or livemark containers:
		var bmID = PlacesUtils.getMostRecentBookmarkForURI(uri);
		if(bmID != -1) {
			return this._getBookmarkDesc(bmID);
		}
		return null;
	},

	setPageNote: function(uri, value) {
		if(value && !BS.isBookmarked(uri)) {
			// Make sure the given bookmarksFolder has not been removed in the meantime:
			if(!this._isBookmarkFolder(this.bookmarksFolderID)) {
				this.bookmarksFolderID = BS.unfiledBookmarksFolder;
			}
			var bmID = BS.insertBookmark(
				this.bookmarksFolderID,
				uri,
				-1,
				HS.getPageTitle(uri)
			);
			this._setBookmarkDesc(bmID, value);
		} else {
			// Set the note as description of all bookmarks
			// (excluding items under tag or livemark containers) for this uri.
			// This ensures it is synced properly via Weave:
			PlacesUtils.getBookmarksForURI(uri).forEach(
				function(bmID) {
					this._setBookmarkDesc(bmID, value);
				},
				this
			);
		}
		if(this.pageNoteTags) {
			if(!value) {
				// Remove the automatically added pageNoteTags:
				TS.untagURI(
					uri,
					this.pageNoteTags
				);
			} else {
				// Tag this uri with the the given pageNoteTags:
				TS.tagURI(
					uri,
					this.pageNoteTags
				);
			}
		}
	},
	
	_isBookmarkFolder: function(itemID) {
		try{
			if(BS.getItemType(itemID) == TYPE_FOLDER) {
				return true;
			}
		}catch(e){
			// Catch NS_ERROR_ILLEGAL_VALUE
		}
		return false;
	},
	
	_getBookmarkDesc: function(bmID) {
		try{
			return	AS.getItemAnnotation(
						bmID,
						DESCRIPTION_ANNO
					);
		}catch(e){
			return null;
		}
	},
	
	_setBookmarkDesc: function(bmID, value) {
		if(!value) {
			// If the value is empty, remove the descrption annotation:
			AS.removeItemAnnotation(
				bmID,
				DESCRIPTION_ANNO
			);
		} else {
			AS.setItemAnnotation(
				bmID,
				DESCRIPTION_ANNO,
				value,
				0,
				EXPIRE_NEVER
			);	
		}
		// Set the last modified time of the bookmark to the current date in microseconds:
		BS.setItemLastModified(bmID, (new Date()).getTime()*1000);
	}
	
}